#!/usr/bin/env python3

"""
    Copyright 2023 carlkidcrypto, All rights reserved.
"""


import unittest
import requests_mock
import sys

sys.path.append("../")

from purpleair_api.PurpleAirAPI import PurpleAirAPI, PurpleAirAPIError


class PurpleAirAPITest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_purpleairapi_valid_read_key(self):
        """
        Test that we can create a PurpleAirAPI with a valid read key
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/keys"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"api_version" : "1.1.1", "time_stamp": 123456789, "api_key_type": "READ"}',
                status_code=200,
            )
            PurpleAirAPI(your_api_read_key="123456789")

    def test_purpleairapi_invalid_read_key(self):
        """
        Test that we can't create a PurpleAirAPI with an invalid read key
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/keys"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"api_version" : "1.1.1", "time_stamp": 123456789, "api_key_type": "WRITE"}',
                status_code=200,
            )

            with self.assertRaises(PurpleAirAPIError):
                PurpleAirAPI(your_api_read_key="123456789")

    def test_purpleairapi_valid_write_key(self):
        """
        Test that we can create a PurpleAirAPI with a valid write key
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/keys"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"api_version" : "1.1.1", "time_stamp": 123456789, "api_key_type": "WRITE"}',
                status_code=200,
            )
            PurpleAirAPI(your_api_write_key="123456789")

    def test_purpleairapi_invalid_write_key(self):
        """
        Test that we can't create a PurpleAirAPI with an invalid write key
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/keys"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"api_version" : "1.1.1", "time_stamp": 123456789, "api_key_type": "READ"}',
                status_code=200,
            )

            with self.assertRaises(PurpleAirAPIError):
                PurpleAirAPI(your_api_write_key="123456789")

    def test_purpleairapi_with_no_args(self):
        """
        Test that we can't create a PurpleAirAPI with no args.
        """

        with self.assertRaises(PurpleAirAPIError):
            PurpleAirAPI()

    def test_purpleairapi_with_two_args(self):
        """
        Test that we can create a PurpleAirAPI with all three args.
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/keys"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"api_version" : "1.1.1", "time_stamp": 987654321, "api_key_type": "WRITE"}',
                status_code=200,
            )

            PurpleAirAPI(
                your_api_write_key="123456789", your_ipv4_address=["192.168.1.2"]
            )

        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"api_version" : "1.1.1", "time_stamp": 987654321, "api_key_type": "READ"}',
                status_code=200,
            )

            PurpleAirAPI(
                your_api_read_key="123456789", your_ipv4_address=["192.168.1.2"]
            )

    def test_purpleairapi_getters(self):
        """
        Test the PurpleAirAPI getters.
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/keys"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"api_version" : "1.1.1", "time_stamp": 987654321, "api_key_type": "WRITE"}',
                status_code=200,
            )

            paa = PurpleAirAPI(your_api_write_key="123456789")

        self.assertEqual(paa.get_api_key_last_checked["123456789"], 987654321)
        self.assertEqual(paa.get_api_key_type["123456789"], "WRITE")
        self.assertEqual(paa.get_api_versions["123456789"], "1.1.1")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

"""
    Copyright 2023 carlkidcrypto, All rights reserved.
"""


import unittest
import sys

sys.path.append("../")

from purpleair_api.PurpleAirAPIError import PurpleAirAPIError


class PurpleAirAPIErrorTest(unittest.TestCase):
    def setUp(self):
        self.error_msg_str = ""

    def tearDown(self):
        self.error_msg_str = ""

    def test_custom_error(self):
        """
        Test that our custom error is called PurpleAirAPIError
        """

        # Setup
        self.error_msg_str = "This is a test!"

        # Action
        retval = PurpleAirAPIError(self.error_msg_str)

        # Expected Result
        self.assertEqual(retval.message, self.error_msg_str)
        self.assertEqual(retval.__class__, PurpleAirAPIError)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

"""
    Copyright 2023 carlkidcrypto, All rights reserved.
    A python3 class designed to fetch data from Purple Air's new API.
    This class will handle all `local` requests
    https://api.purpleair.com/#api-welcome
"""

from purpleair_api.PurpleAirAPIError import PurpleAirAPIError
from purpleair_api.PurpleAirAPIHelpers import send_url_get_request


class PurpleAirLocalAPI:
    """
    The PurpleAirLocalAPI class designed to send valid
    local network requests. It can work with one or many IPv4 addresses.

    :param list ipv4_address_list: A list of strings with valid IPv4 addresses for your sensors. The addresses don't need a CIDR.
    """

    def __init__(self, ipv4_address_list=None):
        # Create the vase API request string for local networks.
        error_msg = (
            "Must provide the IPv4 address list for the sensor(s) on your local network"
        )

        if type(ipv4_address_list) is type(None):
            raise PurpleAirAPIError(error_msg)

        elif type(ipv4_address_list) is not type(list()):
            raise PurpleAirAPIError(error_msg)

        for address in ipv4_address_list:
            if len(address) == 0 or len(address) > 15:
                raise PurpleAirAPIError(error_msg)

        self._base_api_local_network_request_string_dict = dict()
        for address in ipv4_address_list:
            self._base_api_local_network_request_string_dict[address] = (
                f"http://{address}/json"
            )

    def request_local_sensor_data(self) -> dict:
        """
        A method to request a local sensors data. This sensor must be in a netork that is accessible
        """

        retval = {}
        for key, value in self._base_api_local_network_request_string_dict.items():
            request_value = send_url_get_request(value)
            retval[key] = request_value

        return retval



#!/usr/bin/env python3

"""
    Copyright 2023 carlkidcrypto, All rights reserved.
    A python3 class designed to fetch data from Purple Air's new API.
    https://api.purpleair.com/#api-welcome
"""


class PurpleAirAPIError(Exception):
    """
    Custom Exception for our PurpleAirAPI class.
    """

    def __init__(self, message_string):
        self.message = message_string
        super().__init__(self.message)



#!/usr/bin/env python3

"""
    Copyright 2023 carlkidcrypto, All rights reserved.
"""


import unittest
import requests_mock
import sys

sys.path.append("../")

from purpleair_api.PurpleAirReadAPI import PurpleAirReadAPI


class PurpleAirReadAPITest(unittest.TestCase):
    def setUp(self):
        self.para = PurpleAirReadAPI(123456789)

    def tearDown(self):
        self.para = None

    def test_request_sensor_data_with_no_optional_parameters(self):
        """
        Test that we can request a sensors data with no read key and no fields
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/sensors/1234"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.para.request_sensor_data(1234)

    def test_request_sensor_data_with_read_key_no_fields(self):
        """
        Test that we can request a sensors data with a read key and no fields
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/sensors/1234?read_key=56789"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.para.request_sensor_data(1234, 56789)

    def test_request_sensor_data_with_read_key_and_fields(self):
        """
        Test that we can request a sensors data with a read key and fields
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/sensors/1234?read_key=56789&fields=test1,test2,test3"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.para.request_sensor_data(1234, 56789, "test1, test2, test3")

    def test_request_multiple_sensors_data_with_no_optional_parameters(self):
        """
        Test that we can request multiple sensors data with fields
        """

        # Setup
        fake_url_request = (
            "https://api.purpleair.com/v1/sensors/?fields=test1,test2,test3,test4,test5"
        )

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.para.request_multiple_sensors_data(
                fields="test1, test2, test3, test4, test5"
            )

    def test_request_multiple_sensors_data_with_some_optional_parameters(self):
        """
        Test that we can request multiple sensors data with fields, location_type, show_only, max_age, nwlat, selat
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/sensors/?fields=test1,test2,test3,test4,test5&location_type=1&show_only=123,456&max_age=1234&nwlat=5678&selat=91234"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.para.request_multiple_sensors_data(
                fields="test1, test2, test3, test4, test5",
                location_type=1,
                show_only="123, 456",
                max_age=1234,
                nwlat=5678,
                selat=91234,
            )

    def test_request_multiple_sensors_data_with_all_optional_parameters(self):
        """
        Test that we can request multiple sensors data with fields, location_type, read_keys, show_only, modified_since, max_age, nwlng, nwlat, selng, & selat.
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/sensors/?fields=test1,test2,test3,test4,test5&location_type=5&read_keys=asdf&show_only=124,567&modified_since=123456789&max_age=123456789&nwlng=123456&nwlat=6789&selng=444&selat=333"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.para.request_multiple_sensors_data(
                fields="test1, test2, test3, test4, test5",
                location_type=5,
                read_keys="asdf",
                show_only="124, 567",
                modified_since=123456789,
                max_age=123456789,
                nwlng=123456,
                nwlat=6789,
                selng=444,
                selat=333,
            )

    def test_request_sensor_historic_data_with_no_optional_parameters(self):
        """
        Test that we can request historic sensor data with no optional parameters. Just sensor_index and fields.
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/sensors/1234/history?fields=name,field1,field2,etc"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.para.request_sensor_historic_data(
                sensor_index=1234,
                fields="name, field1, field2, etc",
            )

    # def test_request_group_detail_data(self):
    #     pass

    # def test_request_member_data(self):
    #     pass

    # def test_request_member_historic_data(self):
    #     pass

    # def test_request_members_data(self):
    #     pass

    # def test_request_local_sensor_data_none_ip_provided(self):
    #     """
    #     Test that providing no IP address results in a `PurpleAirAPIError`
    #     """

    #     # Setup,  Action, and Expected Result
    #     with self.assertRaises(PurpleAirAPIError):
    #         self.para = PurpleAirLocalAPI(None)

    # def test_request_local_sensor_data_empty_string_ip_provided(self):
    #     """
    #     Test that providing no IP address results in a `PurpleAirAPIError`
    #     """

    #     # Setup,  Action, and Expected Result
    #     with self.assertRaises(PurpleAirAPIError):
    #         self.para = PurpleAirLocalAPI(None)

    #     with self.assertRaises(PurpleAirAPIError):
    #         self.para = PurpleAirLocalAPI("")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

"""
    Copyright 2023 carlkidcrypto, All rights reserved.
"""


import unittest
import requests_mock
import sys

sys.path.append("../")

from purpleair_api.PurpleAirWriteAPI import PurpleAirWriteAPI


class PurpleAirWriteAPITest(unittest.TestCase):
    def setUp(self):
        self.pawa = PurpleAirWriteAPI(123456789)

    def tearDown(self):
        self.pawa = None

    def test_post_create_group_data(self):
        """
        Test that we can post to create a group for sensors.
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/groups"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.post(
                fake_url_request,
                text='{"test" : 1234}',
                status_code=200,
            )
            self.pawa.post_create_group_data("this_is_a_name")

    def test_post_create_member_with_sensor_index(self):
        """
        Test that we can post to add members to a sensor group.
        """

        # Setup
        fake_url_request = "https://api.purpleair.com/v1/groups/1234/members"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.post(
                fake_url_request,
                text='{"test" : 1234}',
                status_code=200,
            )
            self.pawa.post_create_member(group_id=1234, sensor_index=4567)


if __name__ == "__main__":
    unittest.main()
